class CTimeMgr
  {
private:
   static const int SECONDS_IN_DAY;
public:
   MqlDateTime      curr;
   datetime         currDt;
   datetime         todayStartDt;
   int              currMinsOfDay;
   void             update();
   //---
   static bool      stringToMinsOfDay(const string inString, int &result);
   static bool      stringToStruct(const string inString, MqlDateTime &var);
   static int       structToMinsOfDay(const MqlDateTime &inStruct);
   static bool      dtToStruct(datetime a_dt, MqlDateTime &var);
   static bool      dtToMinsOfDay(datetime a_dt, int &result);
  };

const int CTimeMgr::SECONDS_IN_DAY = 86400;

void CTimeMgr::update(void)
  {
   currDt = TimeCurrent(curr);
   currMinsOfDay = structToMinsOfDay(curr);
   todayStartDt = (currDt / SECONDS_IN_DAY) * SECONDS_IN_DAY;
  }

bool CTimeMgr::dtToMinsOfDay(datetime a_dt,int &result)
  {
   MqlDateTime inStruct;
   if(!dtToStruct(a_dt, inStruct))
      return(false);
   result = structToMinsOfDay(inStruct);
   return(true);
  }

bool CTimeMgr::stringToMinsOfDay(const string inString,int &result)
  {
   MqlDateTime inStruct;
   if(!CTimeMgr::stringToStruct(inString, inStruct))
     {
      result = -1;
      return(false);
     }
   result = CTimeMgr::structToMinsOfDay(inStruct);
   return(true);
  }

int CTimeMgr::structToMinsOfDay(const MqlDateTime &inStruct)
  {
   return(inStruct.hour * 60 + inStruct.min);
  }

bool CTimeMgr::stringToStruct(const string inString,MqlDateTime &var)
  {
   if(!dtToStruct(StringToTime(inString), var))
      return(false);
   string back = IntegerToString(var.hour, 2, '0') + ":" + IntegerToString(var.min, 2, '0');
   if(inString == back)
      return(true);
   PrintFormat("Error converting time from string to MqlDateTime. \"%s\" converted to \"%s\"", inString, back);
   return(false);
  }

bool CTimeMgr::dtToStruct(datetime a_dt,MqlDateTime &var)
  {
   ResetLastError();
   if(TimeToStruct(a_dt, var))
      return(true);
   PrintFormat("%s error %i, value %I64i", __FUNCTION__, GetLastError(), a_dt);
   return(false);
  }

//+-----------------------------------------------------------------------------------+
//| isWithin() returns false only if 'm_isEnabled' and 'a_timeInMinutes' out of range |
//+-----------------------------------------------------------------------------------+
class CTimeRange
  {
protected:
   bool        m_isEnabled;
   int         m_minutesFrom;
   int         m_minutesTo;
   bool        isWithin(int a_timeInMinutes) const { return(m_isEnabled ? isTimeInRange(a_timeInMinutes, m_minutesFrom, m_minutesTo) : true); }
   static bool isTimeInRange(int timeInMin, int startInMin, int endInMin);
public:
   bool        initialize(bool a_isEnabled, string a_strTimeFrom, string a_strTimeTo);
  };

class CTimeFilter : public CTimeRange
  {
private:
   bool m_flag;
public:
   void onTick(int currTimeInMinsOfDay) { m_flag = isWithin(currTimeInMinsOfDay); }
   bool doesAllow() const               { return(m_flag);                         }
  };

bool CTimeRange::initialize(bool a_isEnabled,string a_strTimeFrom,string a_strTimeTo)
  {
   m_isEnabled = a_isEnabled;
   if(!m_isEnabled)
      return(true);
   if(!CTimeMgr::stringToMinsOfDay(a_strTimeFrom, m_minutesFrom) || !CTimeMgr::stringToMinsOfDay(a_strTimeTo, m_minutesTo))
      return(false);
   if(m_minutesFrom == m_minutesTo)
     {
      Print(__FUNCTION__, " Time range limits must not match!");
      return(false);
     }
   return(true);
  }

bool CTimeRange::isTimeInRange(int timeInMin,int startInMin,int endInMin)
  {
   if(startInMin > endInMin)
      return(timeInMin < startInMin && timeInMin > endInMin ? false : true);
   else return(timeInMin < startInMin || timeInMin > endInMin ? false : true);
  }
//+------------------------------------------------------------------+
//|                                                     Fractals.mq5 |
//|                             Copyright 2000-2023, MetaQuotes Ltd. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2000-2023, MetaQuotes Ltd."
#property link      "https://www.mql5.com"
//--- indicator settings
#property indicator_chart_window
#property indicator_buffers 2
#property indicator_plots   2
#property indicator_type1   DRAW_ARROW
#property indicator_type2   DRAW_ARROW
#property indicator_color1  Gray
#property indicator_color2  Gray
#property indicator_label1  "Fractal Up"
#property indicator_label2  "Fractal Down"

      bool   inpUseTimeFilter = true;    // Use
input string inpFrom          = "09:00"; // From
input string inpTo            = "23:00"; // To

CTimeMgr    timeMgr;
CTimeFilter timeFilter;

//--- indicator buffers
double ExtUpperBuffer[];
double ExtLowerBuffer[];
//--- 10 pixels upper from high price
int    ExtArrowShift=-10;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
   SetIndexBuffer(0,ExtUpperBuffer,INDICATOR_DATA);
   SetIndexBuffer(1,ExtLowerBuffer,INDICATOR_DATA);
   IndicatorSetInteger(INDICATOR_DIGITS,_Digits);
//--- sets first bar from what index will be drawn
   PlotIndexSetInteger(0,PLOT_ARROW,217);
   PlotIndexSetInteger(1,PLOT_ARROW,218);
//--- arrow shifts when drawing
   PlotIndexSetInteger(0,PLOT_ARROW_SHIFT,ExtArrowShift);
   PlotIndexSetInteger(1,PLOT_ARROW_SHIFT,-ExtArrowShift);
//--- sets drawing line empty value--
   PlotIndexSetDouble(0,PLOT_EMPTY_VALUE,EMPTY_VALUE);
   PlotIndexSetDouble(1,PLOT_EMPTY_VALUE,EMPTY_VALUE);
   if(!timeFilter.initialize(inpUseTimeFilter, inpFrom, inpTo))
      return(INIT_FAILED);
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//|  Fractals on 5 bars                                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
   if(rates_total<5)
      return(0);

   int start;
//--- clean up arrays
   if(prev_calculated<7)
     {
      start=2;
      ArrayInitialize(ExtUpperBuffer,EMPTY_VALUE);
      ArrayInitialize(ExtLowerBuffer,EMPTY_VALUE);
     }
   else
      start=rates_total-5;
//--- main cycle of calculations
   for(int i=start; i<rates_total-3 && !IsStopped(); i++)
     {
      //--- Upper Fractal
      if(high[i]>high[i+1] && high[i]>high[i+2] && high[i]>=high[i-1] && high[i]>=high[i-2])
         ExtUpperBuffer[i]=high[i];
      else
         ExtUpperBuffer[i]=EMPTY_VALUE;

      //--- Lower Fractal
      if(low[i]<low[i+1] && low[i]<low[i+2] && low[i]<=low[i-1] && low[i]<=low[i-2])
         ExtLowerBuffer[i]=low[i];
      else
         ExtLowerBuffer[i]=EMPTY_VALUE;
     }
//---
   timeMgr.update();
   timeFilter.onTick(timeMgr.currMinsOfDay);
   printNewFractal(rates_total, prev_calculated, time);
//--- OnCalculate done. Return new prev_calculated.
   return(rates_total);
  }

void printNewFractal(int rates_total, int prev_calculated, const datetime &time[])
  {
   if(!timeFilter.doesAllow() || rates_total - prev_calculated != 1)
      return;
   int idx = rates_total - 4;
   if(ExtUpperBuffer[idx] != EMPTY_VALUE)
      Print("A new upper fractal ", DoubleToString(ExtUpperBuffer[idx], Digits()), " appeared on the ", TimeToString(time[idx]), " bar");
   if(ExtLowerBuffer[idx] != EMPTY_VALUE)
      Print("A new lower fractal ", DoubleToString(ExtLowerBuffer[idx], Digits()), " appeared on the ", TimeToString(time[idx]), " bar");
  }